"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
const trie_1 = require("./trie");
const trieRef_1 = require("./trieRef");
const gensequence_1 = require("gensequence");
const Rx = require("rxjs/Rx");
function leaves(node) {
    function* walk(node, k, p) {
        if (!node.c) {
            yield { n: node, p, k };
        }
        else {
            const children = [...node.c];
            for (const n of children) {
                yield* walk(n[1], n[0], node);
            }
        }
    }
    return gensequence_1.genSequence(walk(node, ''));
}
function flattenToReferences(node) {
    function* walk() {
        let iterations = 100;
        let processed = 0;
        let index = 0;
        function hash(node) {
            const flags = node.f ? '*' : '';
            const refs = node.r ? '{' + [...node.r].sort((a, b) => a[0] < b[0] ? -1 : 1).map(a => a.join(':')).join(',') + '}' : '';
            return flags + refs;
        }
        do {
            processed = 0;
            let hashMap = new Map();
            for (const leaf of leaves(node)) {
                const h = hash(leaf.n);
                let m = hashMap.get(h);
                if (m === undefined) {
                    // first time, add it to hash
                    yield leaf.n;
                    m = index;
                    hashMap.set(h, m);
                    index += 1;
                }
                // Fix up the parent
                if (leaf.p && leaf.p.c) {
                    leaf.p.r = leaf.p.r || new trieRef_1.RefMap();
                    leaf.p.r.set(leaf.k, m);
                    leaf.p.c.delete(leaf.k);
                    if (!leaf.p.c.size) {
                        delete leaf.p.c;
                    }
                }
                processed += 1;
            }
            iterations -= 1;
        } while (processed && iterations && node.c);
        yield node;
    }
    return gensequence_1.genSequence(walk());
}
const regExpEscapeChars = /([\[\]\\,:{}*])/;
const regExTrailingComma = /,(\}|\n)/g;
function escapeChar(char) {
    return char.replace(regExpEscapeChars, '\\$1');
}
function trieToExportString(node, base = 16) {
    function* walk(node) {
        if (node.f) {
            yield '*';
        }
        if (node.c) {
            yield '[';
            for (const n of node.c) {
                yield escapeChar(n[0]) + '[';
                yield* walk(n[1]);
                yield ']';
            }
            yield ']';
        }
        if (node.r) {
            const refs = [...node.r].sort((a, b) => a[0] < b[0] ? -1 : 1);
            for (const n of refs) {
                const [c, r] = n;
                const ref = r ? r.toString(base) : '';
                yield escapeChar(c) + ref + ',';
            }
        }
    }
    return gensequence_1.genSequence(walk(node));
}
exports.trieToExportString = trieToExportString;
function generateHeader(base) {
    const header = [
        'TrieXv1',
        'base=' + base,
    ];
    return gensequence_1.genSequence(header)
        .map(a => a + '\n');
}
function exportTrie(node, base = 16) {
    const rows = flattenToReferences(node)
        .map(node => {
        const row = [
            ...trieToExportString(node, base),
            '\n',
        ]
            .join('').replace(regExTrailingComma, '$1');
        return row;
    });
    return generateHeader(base)
        .concat(rows);
}
exports.exportTrie = exportTrie;
function importTrieRx(lines) {
    const headerLines = new Rx.Subject();
    let radix = 16;
    headerLines
        .take(2)
        .map(a => a.trim())
        .toArray()
        .subscribe(headerRows => {
        const header = headerRows.join('\n');
        const headerReg = /^TrieXv1\nbase=(\d+)$/;
        if (!headerReg.test(header))
            throw new Error('Unknown file format');
        radix = Number.parseInt(header.replace(headerReg, '$1'), 10);
    });
    const regNotEscapedCommas = /(^|[^\\]),/g;
    const regUnescapeCommas = /__COMMA__/g;
    const regUnescape = /[\\](.)/g;
    function splitLine(line) {
        const pattern = '$1__COMMA__';
        return line
            .replace(regNotEscapedCommas, pattern)
            .split(regUnescapeCommas)
            .map(a => a.replace(regUnescape, '$1'));
    }
    function decodeLine(line, nodes) {
        const isWord = line[0] === '*';
        line = isWord ? line.slice(1) : line;
        const flags = isWord ? { f: trie_1.FLAG_WORD } : {};
        const children = splitLine(line)
            .filter(a => !!a)
            .map(a => [
            a[0],
            Number.parseInt((a.slice(1) || '0'), radix),
        ])
            .map(([k, i]) => [k, nodes[i]]);
        const cNode = children.length ? { c: new trie_1.ChildMap(children) } : {};
        return __assign({}, cNode, flags);
    }
    const r = lines
        .do(headerLines)
        .map(a => a.trim())
        .skipWhile(line => line !== '*')
        .filter(a => !!a)
        .reduce((acc, line) => {
        const { lines, nodes } = acc;
        const root = decodeLine(line, nodes);
        nodes[lines] = root;
        return { lines: lines + 1, root, nodes };
    }, { lines: 0, nodes: [], root: {} })
        .do(r => {
        // console.log(r.lines);
    })
        .map(r => r.root);
    return r;
}
exports.importTrieRx = importTrieRx;
//# sourceMappingURL=importExport.js.map