"use strict";
const gensequence_1 = require("gensequence");
const trie_1 = require("./trie");
function insert(text, node = {}) {
    if (text.length) {
        const head = text[0];
        const tail = text.slice(1);
        node.c = node.c || new trie_1.ChildMap();
        node.c.set(head, insert(tail, node.c.get(head)));
    }
    else {
        node.f = (node.f || 0) | trie_1.FLAG_WORD;
    }
    return node;
}
exports.insert = insert;
/**
 * Sorts the nodes in a trie in place.
 */
function orderTrie(node) {
    if (!node.c)
        return;
    const nodes = [...node.c].sort(([a], [b]) => a < b ? -1 : 1);
    node.c = new Map(nodes);
    for (const n of node.c) {
        orderTrie(n[1]);
    }
}
exports.orderTrie = orderTrie;
/**
 * Generator an iterator that will walk the Trie parent then children in a depth first fashion that preserves sorted order.
 */
function iterateTrie(node) {
    function* iterate(node, text, depth) {
        const r = { node, text, depth };
        yield r;
        if (node.c) {
            for (const n of node.c) {
                const [t, c] = n;
                yield* iterate(c, text + t, depth + 1);
            }
        }
    }
    return gensequence_1.genSequence(iterate(node, '', 0));
}
exports.iterateTrie = iterateTrie;
/**
 * Generate a Iterator that can walk a Trie and yield the words.
 */
function iteratorTrieWords(node) {
    return gensequence_1.genSequence(iterateTrie(node))
        .filter(r => ((r.node.f || 0) & trie_1.FLAG_WORD) === trie_1.FLAG_WORD)
        .map(r => r.text);
}
exports.iteratorTrieWords = iteratorTrieWords;
//# sourceMappingURL=util.js.map